"use strict";
var path_1 = require("path");
var os_1 = require("os");
var ramda_1 = require("ramda");
var DependencyGraph = require('dependency-graph').DepGraph;
function buildDependencyGraph(packagePaths, circular) {
    if (circular === void 0) { circular = []; }
    var configs = {};
    var graph = {};
    packagePaths.forEach(packageFinder(configs));
    var depGraph = new DependencyGraph();
    var packages = Object.keys(configs).map(function (key) { return configs[key]; });
    var packageNames = packages.map(function (pkg) { return pkg.name; });
    packages.forEach(function (_a) {
        var name = _a.name, config = _a.config;
        var devDependencies = findDependencies(config.devDependencies, packageNames);
        var peerDependencies = findDependencies(config.peerDependencies, packageNames);
        var dependencies = findDependencies(config.dependencies, packageNames);
        var allDependencies = ramda_1.union(ramda_1.union(devDependencies, peerDependencies), dependencies);
        depGraph.addNode(name, configs[name]);
        graph[name] = allDependencies.map(function (depName) { return configs[depName]; });
    });
    packageNames.forEach(function (name) {
        if (circular.indexOf(name) === -1) {
            var deps = graph[name];
            deps.forEach(function (dep) {
                if (circular.indexOf(dep.name) === -1)
                    depGraph.addDependency(name, dep.name);
            });
        }
    });
    return new DepGraph(depGraph, configs, circular);
}
exports.buildDependencyGraph = buildDependencyGraph;
var DepGraph = (function () {
    function DepGraph(depGraph, configs, circular) {
        this.depGraph = depGraph;
        this.configs = configs;
        this.circular = circular;
    }
    DepGraph.prototype.dependenciesOf = function (packageName) {
        return this.depGraph.dependenciesOf(packageName);
    };
    DepGraph.prototype.configOf = function (packageName) {
        return ramda_1.clone(this.depGraph.getNodeData(packageName));
    };
    DepGraph.prototype.paths = function () {
        return this.packages().map(function (pkg) { return pkg.path; });
    };
    DepGraph.prototype.packages = function () {
        var configs = this.configs;
        return this.packageNames().map(function (name) { return configs[name]; });
    };
    DepGraph.prototype.packageNames = function () {
        var _a = this, depGraph = _a.depGraph, circular = _a.circular;
        try {
            return depGraph.overallOrder()
                .filter(function (name) { return circular.indexOf(name) === -1; })
                .concat(circular);
        }
        catch (e) {
            var circularPackage = e.message.split(':')[1].split('->')[0].trim();
            throw new Error(os_1.EOL + os_1.EOL + ("" + e.message) + os_1.EOL + os_1.EOL +
                "Circular dependencies are an advanced use-case and must be handled explicitly." + os_1.EOL +
                "To handle circular dependencies it is required to create a " + os_1.EOL +
                "Northbrook configuration file" + "containing at least:" + os_1.EOL + os_1.EOL +
                "  module.exports = {" + os_1.EOL +
                ("    circular: [ '" + circularPackage + "' ]") + os_1.EOL +
                "  }" + os_1.EOL);
        }
    };
    return DepGraph;
}());
exports.DepGraph = DepGraph;
function packageFinder(configs) {
    return function (path) {
        var pkgJson = ramda_1.clone(require(path_1.join(path, 'package.json')));
        configs[pkgJson.name] = { path: path, name: pkgJson.name, config: pkgJson };
    };
}
function findDependencies(dependencies, packageNames) {
    if (!dependencies)
        return [];
    return Object.keys(dependencies)
        .filter(function (name) { return packageNames.indexOf(name) > -1; });
}
;
//# sourceMappingURL=buildDependencyGraph.js.map